#! /usr/bin/env python

import socket
import md5
from index import rconClient, webPage



# Concats a seed string with the password and returns an ASCII-hex MD5 digest
def digest(seed, pw):
    if not pw: return None
    m = md5.new()
    m.update(seed)
    m.update(pw)
    return m.hexdigest()



# BF2 rcon client
class BF2Client(rconClient):
    availMaps = [
        {'mapname': 'dalian_plant'         , 'mapmodes': ['gpm_cq'], 'mapsizes': ['16', '32', '64']},
        {'mapname': 'daqing_oilfields'     , 'mapmodes': ['gpm_cq'], 'mapsizes': ['16', '32', '64']},
        {'mapname': 'dragon_valley'        , 'mapmodes': ['gpm_cq'], 'mapsizes': ['16', '32', '64']},
        {'mapname': 'fushe_pass'           , 'mapmodes': ['gpm_cq'], 'mapsizes': ['16', '32', '64']},
        {'mapname': 'gulf_of_oman'         , 'mapmodes': ['gpm_cq'], 'mapsizes': ['16', '32', '64']},
        {'mapname': 'kubra_dam'            , 'mapmodes': ['gpm_cq'], 'mapsizes': ['16', '32', '64']},
        {'mapname': 'mashtuur_city'        , 'mapmodes': ['gpm_cq'], 'mapsizes': ['16', '32', '64']},
        {'mapname': 'operation_clean_sweep', 'mapmodes': ['gpm_cq'], 'mapsizes': ['16', '32', '64']},
        {'mapname': 'sharqi_peninsula'     , 'mapmodes': ['gpm_cq'], 'mapsizes': ['16', '32', '64']},
        {'mapname': 'songhua_stalemate'    , 'mapmodes': ['gpm_cq'], 'mapsizes': ['16', '32', '64']},
        {'mapname': 'strike_at_karkand'    , 'mapmodes': ['gpm_cq'], 'mapsizes': ['16', '32', '64']},
        {'mapname': 'zatar_wetlands'       , 'mapmodes': ['gpm_cq'], 'mapsizes': ['16', '32', '64']}]

    settingKeys = [
        'sv.autoBalanceTeam',
        'sv.roundsPerMap',
        'sv.spawnTime']

    def __init__(self, host, port, password):
        self.sock = socket.socket (socket.AF_INET, socket.SOCK_STREAM)
        self.sock.connect((host, port))
        response = self.__getWelcomeResponse()
        # Dig out the seed
        prefix = '### Digest seed: '
        challpos = response.find(prefix)
        if challpos != -1:
            challpos += len(prefix)
            challend = response.find('\n', challpos)
            challenge = response[challpos:challend]
        d = digest(challenge, password)
        response = self.invoke('login %s' %d)
        if not response.startswith('Authentication success'):
            raise ValueError("Authentication failed")

    def __del__(self):
        self.sock.close()

    # Read data until two newlines are encountered
    def __getWelcomeResponse(self):
        result = ""
        while 1:
            data = self.sock.recv(1024)
            result += data
            lflfpos = result.find('\n\n')
            if lflfpos != -1: break
        return result

    def invoke(self, cmd):
        self.sock.send('\x02%s\n' %cmd) # This is a non-interactive command.
        return self.getResponse()

    def getResponse(self):
        result = ""
        done = False
        while not done:
            data = self.sock.recv(1024)
            for c in data:
                if c == '\x04':
                    done = True
                    break
                result += c
        return result



    #retrieve map list
    def retrieveMaps(self):
        maps = []
        result = self.invoke('exec maplist.list')
        for line in result[:-1].split('\n'):
            value = line.split()
            id = int(value[0][:-1])
            mapname = value[1][1:-1]
            mapmode = value[2]
            try:
                mapsize = int(value[3])
            except:
                mapsize = 0
            maps.append({'id': id,
                         'mapname': mapname,
                         'mapmode': mapmode,
                         'mapsize': mapsize})
        return maps

    def reloadMaps(self):
        return self.invoke('exec mapList.load')

    def saveMaps(self):
        return self.invoke('exec mapList.save')

    def removeMaps(self, mapIDs):
        mapIDs.sort(lambda x, y: int(y)-int(x))
        # leave at least one map in the maplist
        mapCount = int(self.invoke('exec maplist.mapcount'))
        if mapCount == len(mapIDs): mapIDs = mapIDs[1:]
        for id in mapIDs:
            self.invoke('exec maplist.remove %s' %id)

    def movetotopMaps(self, mapIDs):
        maps = self.retrieveMaps()
        for i in range(len(maps)-1, 0, -1):
            try:
                mapIDs.index(str(i))
                self.invoke('exec maplist.remove %s' %i)
                map = maps[i]
                self.invoke('exec mapList.insert 0 %s %s %s' %(map['mapname'], map['mapmode'], map['mapsize']))
            except:
                pass

    def appendMap(self, mapname, mapmode, mapsize):
        return self.invoke('exec mapList.append %s %s %s' %(mapname, mapmode, mapsize))

    def currentMap(self):
        i = int(self.invoke('exec admin.currentLevel'))
        maps = self.retrieveMaps()
        try:
            map = maps[i]
        except:
            map = {'id': i,
                    'mapname': 'unknown',
                    'mapmode': '?',
                    'mapsize': '?'}
        return map

    def nextMap(self):
        i = int(self.invoke('exec admin.nextLevel'))
        maps = self.retrieveMaps()
        try:
            map = maps[i]
        except:
            map = {'id': i,
                    'mapname': 'unknown',
                    'mapmode': '?',
                    'mapsize': '?'}
        return map

    # restart current map
    def restartMap(self):
        return self.invoke('exec admin.restartMap')

    # start next map
    def runNextMap(self):
        return self.invoke('exec admin.runNextLevel')

    # set next map
    def setNextMap(self, id):
        return self.invoke('exec admin.nextLevel %s' %id)

    # return map image location
    def getMapImage(self, map):
        return '/maps/bf2/%s.jpg' %map['mapname']



    # retrieve player list
    def retrievePlayers(self):
        players = []
        first = True
        result = self.invoke('exec admin.listPlayers')
        for line in result[:-1].split('\n'):
            value = line.split()
            if first and len(value) > 7:
                first = False
                id = int(value[1])
                playername = value[3]
                remoteip = value[7]
            elif len(value) > 2:
                first = True
                cdkey = value[2]
                players.append({'id': id,
                                'playername': playername,
                                'remoteip': remoteip,
                                'cdkey': cdkey})
        return players

    # retrieve banned player list
    def retrieveBannedPlayers(self):
        bannedPlayers = []
        result = self.invoke('exec admin.listBannedAddresses')
        for line in result[:-1].split('\n'):
            value = line.split()
            if len(value) > 2:
                ip = value[1]
                duration = value[2]
                bannedPlayers.append({'id': ip,
                                      'playername': ip,
                                      'duration': duration})
        result = self.invoke('exec admin.listBannedKeys')
        for line in result[:-1].split('\n'):
            value = line.split()
            if len(value) > 2:
                cdkey = value[1]
                duration = value[2]
                bannedPlayers.append({'id': cdkey,
                                      'playername': cdkey,
                                      'duration': duration})
        return bannedPlayers

    def kickPlayers(self, playerIDs):
        for id in playerIDs:
            self.invoke('exec admin.kickPlayer %s' %id)

    def banPlayers(self, playerIDs, duration = 'round'):
        for id in playerIDs:
            if duration == 'round':
                self.invoke('exec admin.banPlayer %s round' %id)
            else:
                self.invoke('exec admin.banPlayer %s perm' %id)

    def unbanPlayers(self, playerIDs):
        for id in playerIDs:
            self.invoke('exec admin.removeAddressFromBanList %s' %id)
            self.invoke('exec admin.removeKeyFromBanList %s' %id)

    def clearBanList(self):
        self.invoke('exec admin.clearBanList')



    def retrieveSettings(self):
        settings = {}
        for key in self.settingKeys:
            value = self.invoke('exec %s' %key)
            settings[key] = value
        return settings

    def saveSettings(self, settings):
        for key in settings:
            result = self.invoke('exec %s %s' %(key, settings[key]))
        return result



    # send message to all players
    def sayAll(self, message):
        return self.invoke('exec game.sayAll "%s"' %message)



# BF2 web page
class BF2Page(webPage):
    images = webPage.images
    images['logo'] = '/images/bf2.gif'



if __name__ == '__main__':
    page = BF2Page(BF2Client(host='localhost', port=4711, password='Prutskip'))
    page.show()
    del page
